-- PARTICLE EXPERIMENTS
-- Author : Seb Lee-Delisle
-- Blog : www.sebleedelisle.com
-- Company : www.pluginmedia.net
--
-- This work is licensed under a Creative Commons  2.0 License.
 
-- Full details at
-- http:--creativecommons.org/licenses/by/2.0/uk/
 
-- You may re-use this code as you wish, but a credit would be
-- appreciated. And I'd love to see what you do with it!
-- mail me : seb@sebleedelisle.com
 
 
cParticle = gideros.class(EventDispatcher)
 
function cParticle:init(sprite,p)
	self.sprite = sprite			-- sprite for the particle

	--self.sprite:setAlpha(.05)
	local t = {}
	for k,v in pairs(p) do
		t[k] = v
	end
	local life = t["particleLifeSpan"]
	local lifeVar = t["particleLifespanVariance"]
	local sourcePosition = t["sourcePosition"]
	local posVar = t["sourcePositionVariance"]
	local startSize = t["startParticleSize"]
	local startSizeVar = t["startParticleSizeVariance"]
	local endSize = t["finishParticleSize"]
	local endSizeVar = t["FinishParticleSizeVariance"]
	local startSpin = t["rotationStart"]
	local startSpinVar = t["rotationStartVariance"]
	local endSpin = t["rotationEnd"]
	local endSpinVar = t["rotationEndVariance"]
	local angle = t["angle"]
	local angleVar = t["angleVariance"]
	
	
	local start = {
		r = clampf(t["startColor"].red + t["startColorVariance"].red * CCRANDOM_MINUS1_1(),0,1),
		g = clampf(t["startColor"].green + t["startColorVariance"].green * CCRANDOM_MINUS1_1(),0,1),
		b = clampf(t["startColor"].blue + t["startColorVariance"].blue * CCRANDOM_MINUS1_1(),0,1),
		a = clampf(t["startColor"].alpha + t["startColorVariance"].alpha * CCRANDOM_MINUS1_1(),0,1)
	}
	local End = {
		r = clampf(t["finishColor"].red + t["finishColorVariance"].red * CCRANDOM_MINUS1_1(),0,1),
		g = clampf(t["finishColor"].green + t["finishColorVariance"].green * CCRANDOM_MINUS1_1(),0,1),
		b = clampf(t["finishColor"].blue + t["finishColorVariance"].blue * CCRANDOM_MINUS1_1(),0,1),
		a = clampf(t["finishColor"].alpha + t["finishColorVariance"].alpha * CCRANDOM_MINUS1_1(),0,1)
	}

 	self.timeToLive = life * lifeVar * CCRANDOM_MINUS1_1()
	self.timeToLive = MAX(0,self.timeToLive)
	self.timeToLive = 1
	self.pos = {
		x = sourcePosition.x + posVar.x * CCRANDOM_MINUS1_1(),
		y = sourcePosition.y + posVar.y * CCRANDOM_MINUS1_1()
	}
	self.color = start
	self.deltaColor = {
		r = (End.r - start.r) / self.timeToLive,
		g = (End.g - start.g) / self.timeToLive,
		b = (End.b - start.b) / self.timeToLive,
		a = (End.a - start.a) / self.timeToLive
	}
	local startS = startSize + startSizeVar * CCRANDOM_MINUS1_1()
	startS = MAX(0,startS)
	
	
	self.size = startS
	self.deltaSize = 0
	if endSize == -1 then 
		self.deltaSize = 0
	else
		local endS = endSize + endSizeVar * CCRANDOM_MINUS1_1()
		endS = MAX(0,endS)
		self.deltaSize = (endS - startS) / self.timeToLive
	end

	local startA = startSpin + startSpinVar * CCRANDOM_MINUS1_1()
	local endA = endSpin + endSpinVar * CCRANDOM_MINUS1_1()
	self.rotation = startA
	self.deltaRotation = (endA - startA) / self.timeToLive

	self.startPos = ccpMult({x=self.sprite:getX(),y=self.sprite:getY()},1)
	
	local a = CC_DEGREES_TO_RADIANS( angle + angleVar * CCRANDOM_MINUS1_1() )
	self.pType = t["emitterType"]
	local v = { x=math.cos(a),y=math.sin(a)}
	local s = t["speed"] + t["speedVariance"] * CCRANDOM_MINUS1_1()
	local startRadius = t["maxRadius"]+t["maxRadiusVariance"]*CCRANDOM_MINUS1_1()
	local endRadius = t["endRadius"]+t["endRadiusVariance"]*CCRANDOM_MINUS1_1()
	local er = 0
	if endRadius == -1 then er = 0 else er = (endRadius-startRadius)/self.timeToLive end
	self.mode = {
		A = {
			dir = ccpMult(v,s),
			radialAccel = t["radialAcceleration"]+t["radialAccelVariance"]*CCRANDOM_MINUS1_1(),
			tangentialAccel = t["tangentialAcceleration"]+t["tangentialAccelVariance"]*CCRANDOM_MINUS1_1(),
			gravity = t["gravity"]
		},
		B = {
			angle = a,
			degreesPerSecond = CC_DEGREES_TO_RADIANS(t["rotatePerSecond"]+t["rotatePerSecondVariance"]*CCRANDOM_MINUS1_1()),
			radius = startRadius,
			deltaRadius = er
		}
	}

	self.disabled = true			-- for disabling the particle so we can reuse it.
 
	self:enable();
end
 
function cParticle:setVel(xvel, yvel)
	self.xVel = xvel
	self.yVel = yvel
end
 
function cParticle:setScale(scale)
	self.sprite:setScale(scale, scale)
end
 
function cParticle:update(dt)
	if not self.disabled then
	
	self.timeToLive = self.timeToLive - dt
	if self.pType == 1 then
		local tmp = {x=0,y=0}
		local radial = {x=0,y=0}
		local tangential = {x=0,y=0}
		if self.pos.x or self.pos.y then
			radial = ccpNormalize(self.pos)
		end
		tangential = radial
		radial = ccpMult(radial,self.mode.A.radialAccel)
		local newy = tangential.x
		tangential.x = -tangential.y
		tangential.y = newy
		tangential = ccpMult(tangential,self.mode.A.tangentialAccel)
		tmp = ccpAdd(ccpAdd(radial,tangential),self.mode.A.gravity)
		tmp = ccpMult(tmp,dt)
		self.mode.A.dir = ccpAdd(self.mode.A.dir,tmp)
		tmp = ccpMult(self.mode.A.dir,dt)
		self.pos = ccpAdd(self.pos,tmp)
	end
	
	if self.pType == 2 then
	
	end
	
	--color
	self.color.r =  self.color.r + ( self.deltaColor.r * dt )
	self.color.g =  self.color.g + ( self.deltaColor.g * dt )
	self.color.b =  self.color.b + ( self.deltaColor.b * dt )
	self.color.a =  self.color.a + ( self.deltaColor.a * dt )
	self.sprite:setColorTransform(self.color.r,self.color.g,self.color.b,self.color.a)
	
	--size
	self.size = self.size + (self.deltaSize * dt)
	self.size = MAX( 0, self.size)
	self.sprite:setScale(self.size,self.size)
	
	--angle
	self.rotation = self.rotation + (self.deltaRotation * dt)
	self.sprite:setRotation(self.rotation)
	
	--local currentPosition = {x=self.sprite:getX(),y=self.sprite:getY()}
	local newpos = {x=0,y=0}
	local diff = ccpSub({x=0,y=0},self.startPos)
	newpos = ccpSub(self.pos,diff)
	newpos = self.pos
	
	self.sprite:setPosition(newpos.x,newpos.y)

	end
end
 
function cParticle:restart(x, y)
	self:initDefaults()
	self:enable()
	self.sprite:setPosition(x, y)
end
 
function cParticle:updateRotation()
	self.sprite:setRotation(math.atan2(self.yVel, self.xVel))	--Math.atan2(yVel,xVel)*(180/Math.PI)
end
 
function cParticle:disable()
	self.sprite:setVisible(false);
	self.disabled = true;
end
 
function cParticle:enable()
	self.sprite:setVisible(true);
	self.disabled = false;
end

function cParticle:reset(p)
	local t = {}
	for k,v in pairs(p) do
		t[k] = v
	end
	local life = t["particleLifeSpan"]
	local lifeVar = t["particleLifespanVariance"]
	local sourcePosition = t["sourcePosition"]
	local posVar = t["sourcePositionVariance"]
	local startSize = t["startParticleSize"]
	local startSizeVar = t["startParticleSizeVariance"]
	local endSize = t["finishParticleSize"]
	local endSizeVar = t["FinishParticleSizeVariance"]
	local startSpin = t["rotationStart"]
	local startSpinVar = t["rotationStartVariance"]
	local endSpin = t["rotationEnd"]
	local endSpinVar = t["rotationEndVariance"]
	local angle = t["angle"]
	local angleVar = t["angleVariance"]
	
	
	local start = {
		r = clampf(t["startColor"].red + t["startColorVariance"].red * CCRANDOM_MINUS1_1(),0,1),
		g = clampf(t["startColor"].green + t["startColorVariance"].green * CCRANDOM_MINUS1_1(),0,1),
		b = clampf(t["startColor"].blue + t["startColorVariance"].blue * CCRANDOM_MINUS1_1(),0,1),
		a = clampf(t["startColor"].alpha + t["startColorVariance"].alpha * CCRANDOM_MINUS1_1(),0,1)
	}
	local End = {
		r = clampf(t["finishColor"].red + t["finishColorVariance"].red * CCRANDOM_MINUS1_1(),0,1),
		g = clampf(t["finishColor"].green + t["finishColorVariance"].green * CCRANDOM_MINUS1_1(),0,1),
		b = clampf(t["finishColor"].blue + t["finishColorVariance"].blue * CCRANDOM_MINUS1_1(),0,1),
		a = clampf(t["finishColor"].alpha + t["finishColorVariance"].alpha * CCRANDOM_MINUS1_1(),0,1)
	}

 	self.timeToLive = life * lifeVar * CCRANDOM_MINUS1_1()
	self.timeToLive = MAX(0,self.timeToLive)
	self.timeToLive = 1
	self.pos = {
		x = sourcePosition.x + posVar.x * CCRANDOM_MINUS1_1(),
		y = sourcePosition.y + posVar.y * CCRANDOM_MINUS1_1()
	}
	self.color = start
	self.deltaColor = {
		r = (End.r - start.r) / self.timeToLive,
		g = (End.g - start.g) / self.timeToLive,
		b = (End.b - start.b) / self.timeToLive,
		a = (End.a - start.a) / self.timeToLive
	}
	local startS = startSize + startSizeVar * CCRANDOM_MINUS1_1()
	startS = MAX(0,startS)
	
	
	self.size = startS
	self.deltaSize = 0
	if endSize == -1 then 
		self.deltaSize = 0
	else
		local endS = endSize + endSizeVar * CCRANDOM_MINUS1_1()
		endS = MAX(0,endS)
		self.deltaSize = (endS - startS) / self.timeToLive
	end

	local startA = startSpin + startSpinVar * CCRANDOM_MINUS1_1()
	local endA = endSpin + endSpinVar * CCRANDOM_MINUS1_1()
	self.rotation = startA
	self.deltaRotation = (endA - startA) / self.timeToLive

	self.startPos = ccpMult({x=self.sprite:getX(),y=self.sprite:getY()},1)
	
	local a = CC_DEGREES_TO_RADIANS( angle + angleVar * CCRANDOM_MINUS1_1() )
	self.pType = t["emitterType"]
	local v = { x=math.cos(a),y=math.sin(a)}
	local s = t["speed"] + t["speedVariance"] * CCRANDOM_MINUS1_1()
	local startRadius = t["maxRadius"]+t["maxRadiusVariance"]*CCRANDOM_MINUS1_1()
	local endRadius = t["endRadius"]+t["endRadiusVariance"]*CCRANDOM_MINUS1_1()
	local er = 0
	if endRadius == -1 then er = 0 else er = (endRadius-startRadius)/self.timeToLive end
	self.mode = {
		A = {
			dir = ccpMult(v,s),
			radialAccel = t["radialAcceleration"]+t["radialAccelVariance"]*CCRANDOM_MINUS1_1(),
			tangentialAccel = t["tangentialAcceleration"]+t["tangentialAccelVariance"]*CCRANDOM_MINUS1_1(),
			gravity = t["gravity"]
		},
		B = {
			angle = a,
			degreesPerSecond = CC_DEGREES_TO_RADIANS(t["rotatePerSecond"]+t["rotatePerSecondVariance"]*CCRANDOM_MINUS1_1()),
			radius = startRadius,
			deltaRadius = er
		}
	}
end